import { LazyBoxPlotData } from '../internal';
import { isNumberColumn } from '../model';
import { default as BoxPlotColumn } from '../model/BoxPlotColumn';
import { isBoxPlotColumn, isNumbersColumn } from '../model/INumberColumn';
import NumberColumn from '../model/NumberColumn';
import { BOX_PLOT, CANVAS_HEIGHT, DOT } from '../styles';
import { colorOf } from './impose';
import { ERenderMode } from './interfaces';
import { renderMissingCanvas, renderMissingDOM } from './missing';
export function computeLabel(v) {
    if (v == null) {
        return '';
    }
    var f = BoxPlotColumn.DEFAULT_FORMATTER;
    return "min = " + f(v.min) + "\nq1 = " + f(v.q1) + "\nmedian = " + f(v.median) + "\nq3 = " + f(v.q3) + "\nmax = " + f(v.max);
}
var BoxplotCellRenderer = (function () {
    function BoxplotCellRenderer() {
        this.title = 'Box Plot';
    }
    BoxplotCellRenderer.prototype.canRender = function (col, mode) {
        return (isBoxPlotColumn(col) && mode === ERenderMode.CELL || (isNumberColumn(col) && mode !== ERenderMode.CELL));
    };
    BoxplotCellRenderer.prototype.create = function (col, context, _hist, imposer) {
        var sortMethod = col.getSortMethod();
        var sortedByMe = col.isSortedByMe().asc !== undefined;
        var width = context.colWidth(col);
        return {
            template: "<div title=\"\">\n                    <div><div></div><div></div></div>\n                 </div>",
            update: function (n, d) {
                var data = col.getBoxPlotData(d);
                var missing = !data || renderMissingDOM(n, col, d);
                if (missing) {
                    return;
                }
                var label = col.getRawBoxPlotData(d);
                renderDOMBoxPlot(n, data, label, sortedByMe ? sortMethod : '', colorOf(col, d, imposer));
            },
            render: function (ctx, d) {
                if (renderMissingCanvas(ctx, col, d, width)) {
                    return;
                }
                var data = col.getBoxPlotData(d);
                if (!data) {
                    return;
                }
                var scaled = {
                    min: data.min * width,
                    median: data.median * width,
                    q1: data.q1 * width,
                    q3: data.q3 * width,
                    max: data.max * width,
                    outlier: data.outlier ? data.outlier.map(function (d) { return d * width; }) : undefined
                };
                renderBoxPlot(ctx, scaled, sortedByMe ? sortMethod : '', colorOf(col, d, imposer), CANVAS_HEIGHT, 0);
            }
        };
    };
    BoxplotCellRenderer.createAggregatedBoxPlot = function (col, rows, raw) {
        if (raw === void 0) { raw = false; }
        var _a;
        var vs = (_a = []).concat.apply(_a, rows.map(function (r) { return (raw ? col.getRawNumbers(r) : col.getNumber(r)); }));
        return new LazyBoxPlotData(vs);
    };
    BoxplotCellRenderer.prototype.createGroup = function (col, _context, _hist, imposer) {
        var sort = (col instanceof NumberColumn && col.isGroupSortedByMe().asc !== undefined) ? col.getSortMethod() : '';
        return {
            template: "<div title=\"\">\n                    <div><div></div><div></div></div>\n                 </div>",
            update: function (n, _group, rows) {
                if (rows.every(function (row) { return col.isMissing(row); })) {
                    renderMissingDOM(n, col, rows[0]);
                    return;
                }
                n.classList.remove('lu-missing');
                var box, label;
                if (isNumbersColumn(col)) {
                    box = BoxplotCellRenderer.createAggregatedBoxPlot(col, rows);
                    label = BoxplotCellRenderer.createAggregatedBoxPlot(col, rows, true);
                }
                else {
                    box = new LazyBoxPlotData(rows.map(function (row) { return col.getNumber(row); }));
                    label = new LazyBoxPlotData(rows.map(function (row) { return col.getRawNumber(row); }));
                }
                renderDOMBoxPlot(n, box, label, sort, colorOf(col, null, imposer));
            }
        };
    };
    BoxplotCellRenderer.prototype.createSummary = function (col, _comtext, _interactive, imposer) {
        return {
            template: "<div title=\"\">\n                    <div><div></div><div></div></div>\n                 </div>",
            update: function (n, hist) {
                if (hist == null || hist.count === 0) {
                    n.classList.add('lu-missing');
                    return;
                }
                n.classList.remove('lu-missing');
                var sort = (col instanceof NumberColumn && col.isGroupSortedByMe().asc !== undefined) ? col.getSortMethod() : '';
                renderDOMBoxPlot(n, hist, hist, sort, colorOf(col, null, imposer));
            }
        };
    };
    return BoxplotCellRenderer;
}());
export default BoxplotCellRenderer;
function renderDOMBoxPlot(n, data, label, sort, color) {
    n.title = computeLabel(label);
    var whiskers = n.firstElementChild;
    var box = whiskers.firstElementChild;
    var median = whiskers.lastElementChild;
    var leftWhisker = Math.max(data.q1 - 1.5 * (data.q3 - data.q1), data.min);
    var rightWhisker = Math.min(data.q3 + 1.5 * (data.q3 - data.q1), data.max);
    whiskers.style.left = Math.round(leftWhisker * 100) + "%";
    var range = rightWhisker - leftWhisker;
    whiskers.style.width = Math.round(range * 100) + "%";
    box.style.left = Math.round((data.q1 - leftWhisker) / range * 100) + "%";
    box.style.width = Math.round((data.q3 - data.q1) / range * 100) + "%";
    box.style.backgroundColor = color;
    median.style.left = Math.round((data.median - leftWhisker) / range * 100) + "%";
    if (!data.outlier || data.outlier.length === 0) {
        whiskers.dataset.sort = sort;
        if (n.children.length > 1) {
            n.innerHTML = '';
            n.appendChild(whiskers);
        }
        return;
    }
    var outliers = Array.from(n.children).slice(1);
    outliers.slice(data.outlier.length).forEach(function (v) { return v.remove(); });
    for (var i = outliers.length; i < data.outlier.length; ++i) {
        var p = n.ownerDocument.createElement('div');
        outliers.push(p);
        n.appendChild(p);
    }
    data.outlier.forEach(function (v, i) {
        delete outliers[i].dataset.sort;
        outliers[i].style.left = Math.round(v * 100) + "%";
    });
    if (sort === 'min') {
        whiskers.dataset.sort = '';
        outliers[0].dataset.sort = 'min';
    }
    else if (sort === 'max') {
        whiskers.dataset.sort = '';
        outliers[outliers.length - 1].dataset.sort = 'max';
    }
}
function renderBoxPlot(ctx, box, sort, color, height, topPadding) {
    var left = Math.max((box.q1 - 1.5 * (box.q3 - box.q1)), box.min);
    var right = Math.min((box.q3 + 1.5 * (box.q3 - box.q1)), box.max);
    ctx.fillStyle = color || BOX_PLOT.box;
    ctx.strokeStyle = BOX_PLOT.stroke;
    ctx.beginPath();
    ctx.rect(box.q1, 0, box.q3 - box.q1, height);
    ctx.fill();
    ctx.stroke();
    var bottomPos = height - topPadding;
    var middlePos = height / 2;
    ctx.beginPath();
    ctx.moveTo(left, middlePos);
    ctx.lineTo(box.q1, middlePos);
    ctx.moveTo(left, topPadding);
    ctx.lineTo(left, bottomPos);
    ctx.moveTo(box.median, 0);
    ctx.lineTo(box.median, height);
    ctx.moveTo(box.q3, middlePos);
    ctx.lineTo(right, middlePos);
    ctx.moveTo(right, topPadding);
    ctx.lineTo(right, bottomPos);
    ctx.stroke();
    ctx.fill();
    if (sort !== '') {
        ctx.strokeStyle = BOX_PLOT.sort;
        ctx.beginPath();
        ctx.moveTo(box[sort], topPadding);
        ctx.lineTo(box[sort], height - topPadding);
        ctx.stroke();
        ctx.fill();
    }
    if (!box.outlier) {
        return;
    }
    ctx.fillStyle = BOX_PLOT.outlier;
    box.outlier.forEach(function (v) {
        ctx.fillRect(Math.max(v - DOT.size / 2, 0), middlePos - DOT.size / 2, DOT.size, DOT.size);
    });
}
//# sourceMappingURL=BoxplotCellRenderer.js.map